@@ -33,6 +33,7 @@ const (
 	modeSource  = "source"
 	modeConvert = "convert" // only intended for use by gopls
 	modeQuery   = "query"   // only intended for use by gopls
+	modeExtract = "extract" // currently, only binary extraction is supported
 )
 
 func parseFlags(cfg *config, stderr io.Writer, args []string) error {
@@ -87,6 +88,7 @@ var supportedModes = map[string]bool{
 	modeBinary:  true,
 	modeConvert: true,
 	modeQuery:   true,
+	modeExtract: true,
 }
 
 var supportedLevels = map[string]bool{
@@ -123,6 +125,22 @@ func validateConfig(cfg *config) error {
 		if !isFile(cfg.patterns[0]) {
 			return fmt.Errorf("%q is not a file", cfg.patterns[0])
 		}
+	case modeExtract:
+		if cfg.test {
+			return fmt.Errorf("the -test flag is not supported in extract mode")
+		}
+		if len(cfg.tags) > 0 {
+			return fmt.Errorf("the -tags flag is not supported in extract mode")
+		}
+		if len(cfg.patterns) != 1 {
+			return fmt.Errorf("only 1 binary can be extracted at a time")
+		}
+		if cfg.json {
+			return fmt.Errorf("the -json flag must be off in extract mode")
+		}
+		if !isFile(cfg.patterns[0]) {
+			return fmt.Errorf("%q is not a file (source extraction is not supported)", cfg.patterns[0])
+		}
 	case modeConvert:
 		if len(cfg.patterns) != 0 {
 			return fmt.Errorf("patterns are not accepted in convert mode")
